

import Foundation
import UIKit



enum MTFont: String {
    
    case merriweather = "Merriweather-Regular"
    //case SourceSansPro = "SourceSansPro-Regular"
    
    func font(_ size: CGFloat = 16.0) -> UIFont {
        return UIFont(name: self.rawValue, size: size)!
    }
}

// MARK: - Font Loading
let loadCustomFonts: () = {
    let loadedFontMontserrat = UIFont.loadFont(MTFont.merriweather.rawValue)
    if loadedFontMontserrat {
        print("================= LOADED FONTS")
    }
}()

extension UIFont {
    
    static func loadFont(_ name: String) -> Bool {
        let bundle = Bundle.main
        guard let fontPath = bundle.path(forResource: name, ofType: "ttf"),
            let data = try? Data(contentsOf: URL(fileURLWithPath: fontPath)),
            let provider = CGDataProvider(data: data as CFData),
            let font = CGFont(provider)
            else {
                return false
        }
        
        var error: Unmanaged<CFError>?
        
        let success = CTFontManagerRegisterGraphicsFont(font, &error)
        if !success {
            print("Error loading font. Font is possibly already registered.")
            return false
        }
        
        return true
    }
}



public extension UILabel {
    @objc public func pushTransition(_ duration: CFTimeInterval) {
        let animation:CATransition = CATransition()
        animation.timingFunction = CAMediaTimingFunction(name: CAMediaTimingFunctionName.easeInEaseOut)
        animation.type = CATransitionType.push
        animation.subtype = CATransitionSubtype.fromTop
        animation.duration = duration
        self.layer.add(animation, forKey: CATransitionType.push.rawValue)
    }
}

public extension CGRect {
    // MARK: - UIView Extends CGRect with helper properties for positioning and setting dimensions
    
    /// The top coordinate of the rect.
    var top: CGFloat {
        get {
            return origin.y
        }
        set(value) {
            origin.y = value
        }
    }
    
    /// The left-side coordinate of the rect.
    var left: CGFloat {
        get {
            return origin.x
        }
        set(value) {
            origin.x = value
        }
    }
    
    /// The bottom coordinate of the rect. Setting this will change origin.y of the rect according to
    /// the height of the rect.
    var bottom: CGFloat {
        get {
            return origin.y + size.height
        }
        set(value) {
            origin.y = value - size.height
        }
    }
    
    /// The right-side coordinate of the rect. Setting this will change origin.x of the rect according to
    /// the width of the rect.
    var right: CGFloat {
        get {
            return origin.x + size.width
        }
        set(value) {
            origin.x = value - size.width
        }
    }
    
    
    /// The center x coordinate of the rect.
    var centerX: CGFloat {
        get {
            return origin.x + size.width / 2
        }
        set (value) {
            origin.x = value - size.width / 2
        }
    }
    
    /// The center y coordinate of the rect.
    var centerY: CGFloat {
        get {
            return origin.y + size.height / 2
        }
        set (value) {
            origin.y = value - size.height / 2
        }
    }
    
    /// The center of the rect.
    var center: CGPoint {
        get {
            return CGPoint(x: centerX, y: centerY)
        }
        set (value) {
            centerX = value.x
            centerY = value.y
        }
    }
    
}

public extension UIView {
    // MARK: - UIView Rect
    
    /// 高度
    var height: CGFloat {
        get { return frame.size.height }
        set { frame.size.height = newValue }
    }
    /// 宽度
    var width: CGFloat {
        get { return frame.size.width }
        set { frame.size.width = newValue }
    }
    /// 横坐标
    var x: CGFloat {
        get { return frame.origin.x }
        set {
            var frame = self.frame
            frame.origin.x = newValue
            self.frame = frame
        }
    }
    /// Y 坐标
    var y: CGFloat {
        get { return frame.origin.y }
        set {
            var frame = self.frame
            frame.origin.y = newValue
            self.frame = frame
        }
    }
    /// 最小 X
    var minX: CGFloat {
        return frame.minX
    }
    /// 最小 Y
    var minY: CGFloat {
        return frame.minY
    }
    
    /// The horizontal center coordinate of the UIView.
    var centerX: CGFloat {
        get {
            return frame.centerX
        }
        set(value) {
            var frame = self.frame
            frame.centerX = value
            self.frame = frame
        }
    }
    
    /// The vertical center coordinate of the UIView.
    var centerY: CGFloat {
        get {
            return frame.centerY
        }
        set(value) {
            var frame = self.frame
            frame.centerY = value
            self.frame = frame
        }
    }
    /// 最大 X
    var maxX: CGFloat {
        return frame.maxX
    }
    /// 最大 Y
    var maxY: CGFloat {
        return frame.maxY
    }
    
}


public extension UIImage {
    
    /// 根据颜色生成图片
    ///
    /// - Parameter color: 颜色
    /// - Returns: 结果图片
    static func image(withColor color: UIColor) -> UIImage {
        let rect = CGRect(x: 0.0, y: 0.0, width: 1.0, height: 1.0)
        UIGraphicsBeginImageContext(rect.size)
        let context = UIGraphicsGetCurrentContext()
        
        context?.setFillColor(color.cgColor)
        context?.fill(rect)
        
        let image = UIGraphicsGetImageFromCurrentImageContext()
        UIGraphicsEndImageContext()
        
        return image!
    }
}


extension UIView {
    /// 添加普通阴影 Opacity=0.4 Radius=4 Offset=(0,3) color=gray
    func applyPlainShadow() {
        let layer = self.layer
        
        layer.shadowColor = UIColor.gray.cgColor
        layer.shadowOffset = CGSize(width: 0, height: 3)
        layer.shadowOpacity = 0.4
        layer.shadowRadius = 4
    }
    
}


public extension NSObject {
    /// 对象类型
    var className: String {
        return type(of: self).className
    }
    /// 对象类型
    static var className: String {
        return stringFrom(self)
    }
    
}

/// 获取指定对象的类型名称
public func stringFrom(_ aClass: AnyClass) -> String {
    return NSStringFromClass(aClass).components(separatedBy: ".").last!
}


extension UIColor {
    
    /// init method with RGB values from 0 to 255, instead of 0 to 1
    convenience init(r: CGFloat, g: CGFloat, b: CGFloat, a: CGFloat = 1) {
        self.init(red: r / 255.0, green: g / 255.0, blue: b / 255.0, alpha: a)
    }
    
    /// 初始化 hex: 0xf3832d3
    convenience init(hex: Int, alpha: CGFloat = 1.0) {
        let red = CGFloat(CGFloat((hex & 0xFF0000) >> 16)/255.0)
        let green = CGFloat(CGFloat((hex & 0x00FF00) >> 8)/255.0)
        let blue = CGFloat(CGFloat((hex & 0x0000FF) >> 0)/255.0)
        self.init(red: red, green: green, blue: blue, alpha: alpha)
    }

}


public extension Dictionary where Value : Equatable  {
    /// 得到相同Value的键数组
    func allKeysForValue(_ val : Value) -> [Key] {
        return self.filter { $1 == val }.map { $0.0 }
    }
    
    init(_ seq: Zip2Sequence<[Key], [Value]>) {
        self.init()
        for (k,v) in seq {
            self[k] = v
        }
    }
}


