//  Created by Luochun on 2017/2/12.
//  Copyright © 2017年 Mantis. All rights reserved.
//

import UIKit


let STPopupFirstResponderDidChange = Notification.Name(rawValue: "STPopupFirstResponderDidChange")

extension UIResponder {

    @objc func st_becomeFirstResponder() -> Bool {
        let accepted = st_becomeFirstResponder()
        if accepted {
            NotificationCenter.default.post(name: STPopupFirstResponderDidChange, object: self)
        }
        return accepted
    }
}

extension UIViewController: SelfAware {
    private struct AssociatedKeys {
        static var landscapeContentSizeInPopupKey: String? = "landscapeContentSizeInPopup"
        static var contentSizeInPopupKey: String? = "contentSizeInPopup"
        static var popupControllerKey: String? = "popupController"
    }

    static let controllerOnceToken = UUID().uuidString

    public static func awake() {
        DispatchQueue.once(token: controllerOnceToken) {
            let selectors: [(Selector, Selector)] = [
                (#selector(viewDidLoad), #selector(st_viewDidLoad)),
                (#selector(present(_:animated:completion:)), #selector(st_present(_:animated:completion:))),
                (#selector(dismiss(animated:completion:)), #selector(st_dismiss(animated:completion:))),
                (#selector(getter: presentedViewController), #selector(getter: st_presentedViewController)),
                (#selector(getter: presentingViewController), #selector(getter: st_presentingViewController))
            ]
            selectors.forEach {
                swizzle($0.0, to: $0.1)
            }
            
            swizzle(#selector(becomeFirstResponder), to: #selector(st_becomeFirstResponder))
        }
    }


    class func swizzle(_ originalSelector: Selector, to swizzledSelector: Selector) {
        let originalMethod = class_getInstanceMethod(self, originalSelector)
        let swizzledMethod = class_getInstanceMethod(self, swizzledSelector)
        method_exchangeImplementations(originalMethod!, swizzledMethod!)
    }

    @objc func st_viewDidLoad() {
        var contentSize = CGSize.zero
        switch UIApplication.shared.statusBarOrientation {
        case .landscapeLeft, .landscapeRight:
            contentSize = landscapeContentSizeInPopup
            if contentSize == .zero {
                contentSize = contentSizeInPopup
            }
        default:
            contentSize = contentSizeInPopup
        }

        if contentSize != .zero {
            view.frame = CGRect(origin: .zero, size: contentSize)
        }
        st_viewDidLoad()
    }

    @objc func st_present(_ viewControllerToPresent: UIViewController, animated: Bool, completion: (() -> Void)?) {
        guard let popupController = popupController else {
            st_present(viewControllerToPresent, animated: animated, completion: completion)
            return
        }

        //let controller = popupController.value(forKey: "containerViewController") as? UIViewController
        let controller = popupController.containerViewController
        controller?.present(viewControllerToPresent, animated: animated, completion: completion)
    }

    @objc func st_dismiss(animated: Bool, completion: (() -> Void)?) {
        guard let popupController = popupController else {
            st_dismiss(animated: animated, completion: completion)
            return
        }

        popupController.dismiss(with: completion)
    }

    @objc var st_presentedViewController: UIViewController? {
        guard let popupController = popupController else { return self.st_presentedViewController }

        let controller = popupController.containerViewController
        return controller?.presentedViewController
    }

    @objc var st_presentingViewController: UIViewController? {
        guard let popupController = popupController else { return self.st_presentingViewController }
        //let controller = popupController.value(forKey: "containerViewController") as? UIViewController
        let controller = popupController.containerViewController
        return controller?.presentingViewController
    }

    static let screenW = UIScreen.main.bounds.width
    static let screenH = UIScreen.main.bounds.height

    var contentSizeInPopup: CGSize {
        set {
            var value = newValue
            if value != .zero && value.width == 0 {
                switch UIApplication.shared.statusBarOrientation {
                case .landscapeLeft, .landscapeRight:
                    value.width = UIViewController.screenH
                default:
                    value.width = UIViewController.screenW
                }
            }

            objc_setAssociatedObject(self, &AssociatedKeys.contentSizeInPopupKey, NSValue(cgSize: value), .OBJC_ASSOCIATION_RETAIN_NONATOMIC)
        }

        get {
            return (objc_getAssociatedObject(self, &AssociatedKeys.contentSizeInPopupKey) as? CGSize) ?? .zero
        }
    }

    var landscapeContentSizeInPopup: CGSize {
        set {
            var value = newValue
            if value != .zero && value.width == 0 {
                switch UIApplication.shared.statusBarOrientation {
                case .landscapeLeft, .landscapeRight:
                    value.width = UIViewController.screenW
                default:
                    value.width = UIViewController.screenH
                }
            }
            objc_setAssociatedObject(self, &AssociatedKeys.landscapeContentSizeInPopupKey, NSValue(cgSize: value), .OBJC_ASSOCIATION_RETAIN_NONATOMIC)
        }

        get {
            return (objc_getAssociatedObject(self, &AssociatedKeys.landscapeContentSizeInPopupKey) as? CGSize) ?? .zero
        }
    }

    var popupController: STPopupController? {
        set {
            guard let newValue = newValue else { return }
            objc_setAssociatedObject(self, &AssociatedKeys.popupControllerKey, newValue, .OBJC_ASSOCIATION_ASSIGN)
        }

        get {
            let popupController = objc_getAssociatedObject(self, &AssociatedKeys.popupControllerKey) as? STPopupController
            guard let controller = popupController else {
                return parent?.popupController
            }
            return controller
        }
    }
}


import Foundation

/// 定义 `protocol` SelfAware
///
///     /// 必须实现SelfAware协议, 重写awake方法
///     extension UIButton: SelfAware {
///         private static let _onceToken = UUID().uuidString
///
///         static func awake() {
///            DispatchQueue.once(token: _onceToken) {
///                replaceMethod(NSClassFromString("UIButton")!, #selector(setTitle(_:for:)), #selector(mt_setTitle(_:for:)))
///                replaceMethod(NSClassFromString("UIButton")!, #selector(setTitleColor(_:for:)), #selector(mt_setTitleColor(_:for:)))
///            }
///         }
///
///         private func mt_setTitle(_ title: String, for state: UIControlState) {
///              mt_setTitle(title, for: state) // 方法替换之后相当于调用系统的setTitle方法
///             // todo
///         }
///     }
///
public protocol SelfAware: class {
    static func awake()
}

/// 创建代理执行单例
class NothingToSeeHere {
    
    static func harmlessFunction() {
        let typeCount = Int(objc_getClassList(nil, 0))
        let types = UnsafeMutablePointer<AnyClass?>.allocate(capacity: typeCount)
        let autoreleasingTypes = AutoreleasingUnsafeMutablePointer<AnyClass>(types)
        objc_getClassList(autoreleasingTypes, Int32(typeCount)) //获取所有的类
        for index in 0 ..< typeCount { (types[index] as? SelfAware.Type)?.awake() } //如果该类实现了SelfAware协议，那么调用awake方法
        types.deallocate()
        
    }
}


/// Swizzling Selector replaceMethod
///
/// - Parameters:
///   - _class: 类名
///   - _originSelector: 类的函数
///   - _newSelector: 新函数
public func replaceMethod(_ _class: AnyClass, _ _originSelector: Selector, _ _newSelector: Selector) {
    let oriMethod = class_getInstanceMethod(_class, _originSelector)
    let newMethod = class_getInstanceMethod(_class, _newSelector)
    let isAddedMethod = class_addMethod(_class, _originSelector, method_getImplementation(newMethod!), method_getTypeEncoding(newMethod!))
    if isAddedMethod {
        class_replaceMethod(_class, _newSelector, method_getImplementation(oriMethod!), method_getTypeEncoding(oriMethod!))
    } else {
        method_exchangeImplementations(oriMethod!, newMethod!)
    }
}

/// 执行单例
extension UIApplication {
    private static let runOnce: Void = {
        //使用静态属性以保证只调用一次(该属性是个方法)
        NothingToSeeHere.harmlessFunction()
    }()
    
    open override var next: UIResponder? {
        UIApplication.runOnce
        return super.next
    }
}


public extension DispatchQueue {
    private static var _onceTracker = [String]()
    
    
    /// 运行一次
    ///
    /// - Parameters:
    ///   - file: 文件
    ///   - function: 函数
    ///   - line: 行
    ///   - block: 运行事件
    static func once(file: String = #file, function: String = #function, line: Int = #line, block:(()->()) ) {
        let token = file + ":" + function + ":" + String(line)
        once(token: token, block: block)
    }
    
    
    /// Executes a block of code, associated with a unique token, only once.  The code is thread safe and will
    ///    only execute the code once even in the presence of multithreaded calls.
    ///
    /// - Parameters:
    ///   - token: A unique reverse DNS style name such as com.vectorform.<name> or a GUID
    ///   - block: Block to execute once
    static func once(token: String, block:(()->()) ) {
        objc_sync_enter(self)
        defer { objc_sync_exit(self) }
        
        
        if _onceTracker.contains(token) {
            return
        }
        
        _onceTracker.append(token)
        block()
    }
}
